import { Anthropic } from '@anthropic-ai/sdk';
import { a as CompletionRunner, C as CompletionBuilder, M as Merge } from '../completion.builder-Jl9rncqe.cjs';
import { Resolvable } from '@synstack/resolved';
import { L as Llm } from '../llm.types-CPjnMffO.cjs';
import 'zod';

interface Cache {
    fn: <TFnArgs extends any[], TFnOutput>(fn: (...args: TFnArgs) => Promise<TFnOutput> | TFnOutput) => (...args: TFnArgs) => Promise<TFnOutput>;
}

type BasePartial<T> = Partial<T>;
declare namespace AnthropicRunner {
    type Instance<T extends Config> = AnthropicRunner<T>;
    namespace Instance {
        type Ready = CompletionRunner<Completion>;
    }
    interface Config {
        model: Anthropic.Model;
        client: Anthropic;
        retries: number;
        continues: number;
        cache?: Cache;
        retryBackoff?: number;
    }
    namespace Config {
        type Partial = BasePartial<Config>;
    }
    type Completion = Llm.Completion;
    namespace Completion {
        type Partial = BasePartial<Completion>;
    }
}
declare class AnthropicRunner<TConfig extends AnthropicRunner.Config.Partial> implements CompletionRunner<AnthropicRunner.Completion> {
    private readonly _config;
    static readonly MODELS: {
        CLAUDE_3_SMALL: string;
        CLAUDE_3_MEDIUM: string;
        CLAUDE_3_LARGE: string;
        CLAUDE_3_5_MEDIUM: string;
    };
    private constructor();
    static new(this: void): AnthropicRunner<{
        retries: number;
        continues: number;
    }>;
    static from<U extends AnthropicRunner.Config.Partial>(this: void, config: U): AnthropicRunner<{
        continues: number;
        retries: number;
    } & U>;
    client(client: Anthropic): AnthropicRunner<TConfig & {
        client: Anthropic;
    }>;
    retries(retries: number): AnthropicRunner<TConfig & {
        retries: number;
    }>;
    cache(cache?: Cache): AnthropicRunner<TConfig & {
        cache: Cache | undefined;
    }>;
    continues(continues: number): AnthropicRunner<TConfig & {
        continues: number;
    }>;
    retryBackoff(retryBackoff?: number): AnthropicRunner<TConfig & {
        retryBackoff: number | undefined;
    }>;
    model(model: Anthropic.Model): AnthropicRunner<TConfig & {
        model: Anthropic.Messages.Model;
    }>;
    chatCompletion(this: AnthropicRunner<AnthropicRunner.Config>, completion: CompletionBuilder<Resolvable<AnthropicRunner.Completion>>): Promise<{
        message: Llm.Assistant.Message;
        stopReason: Llm.StopReason;
        usage: Llm.Usage;
    }>;
    runChatCompletion<T extends AnthropicRunner.Completion>(this: AnthropicRunner<AnthropicRunner.Config>, completion: CompletionBuilder<Resolvable<T>>): Promise<CompletionBuilder<Merge<T, Llm.Completion.Response.Part>>>;
    private static mapStopReason;
    private static mapResponseMessage;
    private static mapResponseContent;
    mapCompletionQuery(this: AnthropicRunner<{
        model: string;
    }>, query: AnthropicRunner.Completion): Anthropic.Messages.MessageCreateParamsNonStreaming;
    private static mapToolConfig;
    private static mapTool;
    private static mapToolChoice;
    private static mapMessage;
    private static mapMessageContent;
}

export { AnthropicRunner };
