import { z } from 'zod';

type $Partial<T> = Partial<T>;
declare namespace Llm {
    type StopReason = "end" | "max_tokens" | "tool_call" | "stop_sequence";
    type Usage = {
        inputTokens: number;
        outputTokens: number;
    };
    type Tool = {
        name: string;
        schema: z.ZodSchema;
    };
    type Completion = {
        temperature: number;
        maxTokens: number;
        messages: Array<Llm.Message>;
        system?: string;
        topK?: number;
        topP?: number;
        stopSequences?: string[];
        toolsConfig?: Llm.Completion.Tool;
        usage?: Usage;
        stopReason?: StopReason;
    };
    namespace Completion {
        type Tool = Tool.Single | Tool.Multi;
        namespace Tool {
            type Type = "single" | "multi";
            type Single = {
                type: "single";
                tool: Llm.Tool;
            };
            type Multi = {
                type: "multi";
                tools: Array<Llm.Tool>;
                requireToolUse?: boolean;
            };
        }
        type WithResponse = Completion & {
            stopReason: StopReason;
            usage: Usage;
        };
        namespace Response {
            type Part = {
                stopReason: StopReason;
                usage: Usage;
            };
        }
        type Partial = $Partial<Completion>;
    }
    type Message = User.Message | Assistant.Message;
    namespace Message {
        type Role = User.Role | Assistant.Role;
        type Content = Content.Text | Content.Image | Content.ToolCall | Content.ToolResponse;
        namespace Content {
            type Text = {
                type: "text";
                text: string;
            };
            type Image = {
                type: "image";
                image: Image.Base64;
            };
            namespace Image {
                type Base64 = {
                    type: "base64";
                    data: string;
                    mimeType: string;
                };
            }
            type ToolCall = {
                type: "tool_call";
                toolCallId: string;
                toolName: string;
                toolArgs: any;
            };
            type ToolResponse = {
                type: "tool_response";
                toolCallId: string;
                toolOutput: any;
            };
        }
    }
    namespace User {
        type Role = "user";
        type Content = Message.Content.Text | Message.Content.Image | Message.Content.ToolResponse;
        type Message = {
            role: Role;
            content: Array<Content>;
        };
    }
    namespace Assistant {
        type Role = "assistant";
        type Content = Message.Content.Text | Message.Content.ToolCall;
        type Message = {
            role: Role;
            content: Array<Content>;
        };
    }
}

export { Llm as L };
